/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formcycle;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;

import org.assertj.core.data.MapEntry;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;

class FormDataHtmlCleanerTest {

	private static final String KEY = "key";
	private static final Object VALUE = "value";

	@Spy
	@InjectMocks
	private FormDataHtmlCleaner cleaner;

	@Nested
	class TestClean {

		private final static Map<String, Object> FORM_DATA_MAP = Map.of(KEY, VALUE);
		private final static FormData FORM_DATA = FormData.builder().formData(FORM_DATA_MAP).build();

		@Test
		void shouldCallCleanFormData() {
			cleaner.clean(FORM_DATA);

			verify(cleaner).cleanFormData(FORM_DATA_MAP);
		}

		@Test
		void shouldSetCleanedFormData() {
			var cleanedFormData = Map.of(KEY, VALUE);
			doReturn(cleanedFormData).when(cleaner).cleanFormData(anyMap());

			var result = cleaner.clean(FORM_DATA);

			assertThat(result.getFormData()).isSameAs(cleanedFormData);
		}
	}

	@Nested
	class TestCleanFormData {

		@Test
		void shouldCallCleanValue() {
			cleaner.cleanFormData(Map.of(KEY, VALUE));

			verify(cleaner).cleanValue(VALUE);
		}

		@Test
		void shouldReturnCleanedMap() {
			var cleanedValue = "noHtml";
			doReturn(cleanedValue).when(cleaner).cleanValue(any());

			var result = cleaner.cleanFormData(Map.of(KEY, VALUE));

			assertThat(result).containsOnly(MapEntry.entry(KEY, cleanedValue));
		}
	}

	@Nested
	class TestCleanValue {

		@Nested
		class TestCleanMap {

			@Test
			void shouldCallCleanFormData() {
				var expectedMap = Map.of(KEY, VALUE);

				cleaner.cleanValue(expectedMap);

				verify(cleaner).cleanFormData(expectedMap);
			}

			@Test
			void shouldReturnValue() {
				var expectedMap = Map.of(KEY, VALUE);
				doReturn(expectedMap).when(cleaner).cleanFormData(anyMap());

				var result = cleaner.cleanValue(Map.of("a", "b"));

				assertThat(result).isSameAs(expectedMap);
			}
		}

		@Nested
		class TestCleanCollection {

			@Test
			void shouldCallCleanValue() {
				cleaner.cleanValue(List.of(VALUE));

				verify(cleaner).cleanValue(VALUE);
			}

			@Test
			void shouldReturnValue() {
				doReturn(List.of(VALUE)).when(cleaner).cleanValue(any());

				var result = cleaner.cleanValue(List.of("a"));

				assertThat(result).isInstanceOf(List.class).asList().containsExactly(VALUE);
			}
		}

		@Nested
		class TestCleanString {

			@Test
			void shouldCallParseHtml() {
				var stringValue = VALUE.toString();

				cleaner.cleanValue(VALUE);

				verify(cleaner).parseHtml(stringValue);
			}

			@Test
			void shouldReturnValue() {
				var cleanedValue = "noHtml";
				doReturn(cleanedValue).when(cleaner).parseHtml(anyString());

				var result = cleaner.cleanValue(VALUE);

				assertThat(result).isEqualTo(cleanedValue);
			}
		}

		@Test
		void shouldReturnUnmodifiedValue() {
			var value = 1;

			var result = cleaner.cleanValue(value);

			assertThat(result).isEqualTo(1);
			verify(cleaner, never()).parseHtml(any());
		}
	}

	@Nested
	class TestHtmlCleaner {

		static final String KEY_LABEL = "label";
		static final String KEY_VALUE = "value";

		static final Map<String, Object> FORM_DATA_MAP = Map.of("tf1", Map.of(
						KEY_LABEL, "<p><em>&Auml;</em></p>",
						KEY_VALUE, "Ä - Wert"),
				"tf2", Map.of(
						KEY_LABEL, "<p><strong>&Ouml;</strong></p>",
						KEY_VALUE, "Ö - Wert"),
				"fs1", Map.of(
						KEY_LABEL, "Ü",
						KEY_VALUE, Map.of(
								"tf3", Map.of(
										KEY_LABEL, " <p><s>Label mit</s> &szlig;</p>",
										KEY_VALUE, "ein Text mit ß und <html><body><h1>Hello</h1><body><html>")),
						"tf4", Map.of(
								KEY_LABEL, "<p><span style=\"background-color:#1abc9c;\">&auml;</span></p>",
								KEY_VALUE, "Text"),
						"ed1", Map.of(
								KEY_LABEL,
								"<ol>\n\t<li><em><strong><u>&ouml;</u></strong></em></li>\n\t<li><span style=\"color:#e74c3c;\">&uuml;</span></li>\n</ol>",
								KEY_VALUE, "TExt\nmit\n    Leerzeichen\nund\n  Umbrüchen"
						)));

		static final Map<String, Object> EXPECTED_MAP = Map.of("tf1", Map.of(
						KEY_LABEL, "Ä",
						KEY_VALUE, "Ä - Wert"),
				"tf2", Map.of(
						KEY_LABEL, "Ö",
						KEY_VALUE, "Ö - Wert"),
				"fs1", Map.of(
						KEY_LABEL, "Ü",
						KEY_VALUE, Map.of(
								"tf3", Map.of(
										KEY_LABEL, "Label mit ß",
										KEY_VALUE, "ein Text mit ß und Hello")),
						"tf4", Map.of(
								KEY_LABEL, "ä",
								KEY_VALUE, "Text"),
						"ed1", Map.of(
								KEY_LABEL,
								"\n\tö\n\tü\n",
								KEY_VALUE, "TExt\nmit\n    Leerzeichen\nund\n  Umbrüchen"
						)));

		@Test
		void shouldCleanHtml() {
			var result = cleaner.clean(FormData.builder().formData(FORM_DATA_MAP).build());

			assertThat(result.getFormData()).isEqualTo(EXPECTED_MAP);
		}
	}
}